@@ -1,5 +1,5 @@
+#![allow(clippy::non_send_fields_in_send_ty)]
 use parking_lot::{lock_api::RawRwLock as _, RawRwLock, RwLock, RwLockReadGuard, RwLockWriteGuard};
-use send_wrapper::SendWrapper;
 
 #[derive(Debug)]
 pub struct Guard(RwLock<()>);
@@ -12,13 +12,13 @@ impl Guard {
 	pub fn shared(&self) -> SharedGuard {
 		let inner = self.0.read();
 
-		SharedGuard(SendWrapper::new(inner))
+		SharedGuard(inner)
 	}
 
 	pub fn exclusive(&self) -> ExclusiveGuard {
 		let inner = self.0.write();
 
-		ExclusiveGuard(SendWrapper::new(inner))
+		ExclusiveGuard(inner)
 	}
 }
 
@@ -28,6 +28,11 @@ impl Default for Guard {
 	}
 }
 
-pub struct SharedGuard<'a>(SendWrapper<RwLockReadGuard<'a, ()>>);
+// implementing send doesn't matter bc we're not actually editing the value, just using it for a locking mechanism
+pub struct SharedGuard<'a>(RwLockReadGuard<'a, ()>);
 
-pub struct ExclusiveGuard<'a>(SendWrapper<RwLockWriteGuard<'a, ()>>);
+unsafe impl<'a> Send for SharedGuard<'a> {}
+
+pub struct ExclusiveGuard<'a>(RwLockWriteGuard<'a, ()>);
+
+unsafe impl<'a> Send for ExclusiveGuard<'a> {}
