@@ -14,6 +14,9 @@
 #![cfg_attr(not(test), warn(clippy::panic_in_result_fn))]
 //! A simple database system that allows the use of multiple different backends.
 
+#[cfg(feature = "metadata")]
+const METADATA_KEY: &str = "__metadata__";
+
 use std::result::Result as StdResult;
 
 pub mod action;
